SET statement_timeout = 0;

SET lock_timeout = 0;

SET idle_in_transaction_session_timeout = 0;

SET client_encoding = 'UTF8';

SET standard_conforming_strings = ON;

SELECT
    pg_catalog.set_config('search_path', '', FALSE);

SET check_function_bodies = FALSE;

SET xmloption = content;

SET client_min_messages = warning;

SET row_security = OFF;

SET default_tablespace = '';

SET default_table_access_method = heap;


CREATE TABLE ref.company (
    name character varying(100) NOT NULL
);

ALTER TABLE ref.company OWNER TO postgres;


ALTER TABLE ONLY ref.company
    ADD CONSTRAINT ref_company_unique UNIQUE (name);


ALTER TABLE ONLY ref.company
    ADD CONSTRAINT ref_company_pkey PRIMARY KEY (name);