GRANT USAGE ON SCHEMA assay TO PUBLIC;

ALTER DEFAULT PRIVILEGES IN SCHEMA assay GRANT SELECT ON TABLES TO xdb_viewer;
ALTER DEFAULT PRIVILEGES IN SCHEMA assay GRANT SELECT, UPDATE, INSERT ON TABLES TO xdb_logger;
ALTER DEFAULT PRIVILEGES IN SCHEMA assay GRANT ALL ON TABLES TO xdb_admin;


GRANT SELECT ON ALL TABLES IN SCHEMA assay TO xdb_viewer;
GRANT SELECT, UPDATE, INSERT ON ALL TABLES IN SCHEMA assay TO xdb_logger;
GRANT CREATE ON SCHEMA assay TO xdb_admin;
GRANT ALL ON ALL TABLES IN SCHEMA assay TO xdb_admin;
