----------------------------------------------------------------------------
-- xplordb
-- 
-- Copyright (C) 2025  Oslandia / OpenLog
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Affero General Public License as published
-- by the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Affero General Public License for more details.
-- 
-- You should have received a copy of the GNU Affero General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.
-- 
-- __authors__ = ["rdelhome"]
-- __contact__ = "infos@oslandia.com"
-- __date__ = "2025/06/17"
-- __license__ = "AGPLv3"
----------------------------------------------------------------------------

--
-- PostgreSQL database dump
--

-- Dumped from database version 16.9 (Ubuntu 16.9-1.pgdg24.04+1)
-- Dumped by pg_dump version 16.9 (Ubuntu 16.9-1.pgdg24.04+1)

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

SET default_tablespace = '';

SET default_table_access_method = heap;

--
-- Name: assay_column_definition; Type: TABLE; Schema: assay; Owner: xdb_admin
--

CREATE TABLE assay.assay_column_definition (
    name character varying NOT NULL,
    variable character varying NOT NULL,
    col character varying
);


ALTER TABLE assay.assay_column_definition OWNER TO xdb_admin;

--
-- Name: assay_column_definition assay_column_definition_pkey; Type: CONSTRAINT; Schema: assay; Owner: xdb_admin
--

ALTER TABLE ONLY assay.assay_column_definition
    ADD CONSTRAINT assay_column_definition_pkey PRIMARY KEY (name, variable);


--
-- Name: assay_column_definition assay_column_definition_variable_fkey; Type: FK CONSTRAINT; Schema: assay; Owner: xdb_admin
--

ALTER TABLE ONLY assay.assay_column_definition
    ADD CONSTRAINT assay_column_definition_variable_fkey FOREIGN KEY (variable) REFERENCES assay.assay_definition(variable);


--
-- PostgreSQL database dump complete
--
