----------------------------------------------------------------------------
-- xplordb
-- 
-- Copyright (C) 2022  Oslandia / OpenLog
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Affero General Public License as published
-- by the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Affero General Public License for more details.
-- 
-- You should have received a copy of the GNU Affero General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.
-- 
-- __authors__ = ["vlarmet"]
-- __contact__ = "vincent.larmet@apeiron.technology"
-- __date__ = "2024/03/18"
-- __license__ = "AGPLv3"


CREATE TRIGGER display_trace_update
AFTER UPDATE OF geom_trace, planned_trace ON dh.collar
FOR EACH ROW
EXECUTE FUNCTION display.display_trace();


CREATE TRIGGER a_populate_display_table
AFTER INSERT OR DELETE OR UPDATE OF data_set,
hole_id,
eoh,
planned_eoh ON dh.collar
FOR EACH ROW
EXECUTE FUNCTION dh.populate_display_table();